﻿package {
	import flash.display.DisplayObject;
	import flash.events.Event;
	import flash.events.MouseEvent;
	
	import net.alumican.as3.ui.justputplay.scrollbars.JPPScrollbar;
	import net.alumican.as3.ui.justputplay.events.JPPScrollbarEvent;
	import flash.display.Sprite;
	import flash.display.Stage;
	import fl.transitions.Tween;
	import flash.utils.Timer;
	import flash.events.TimerEvent;
	import flash.display.MovieClip;

	
	/**
	 * TouchScrollbarKit
	 *
	 * <p>ActionScript3.0で簡単に設置できるスクロールバー. </p>
	 * <p>適切な階層構造を持ったDisplayObjectにリンケージすることで, 簡単にスクロールバーを設置することができます. </p>
	 * 
	 * @author alumican.net<Yukiya Okuda>
	 * @link http://alumican.net
	 */
	
	public class TouchScrollbarKit {
		
		/** スクロールバークラスです */
		private var _scrollbar:JPPScrollbar;
		
		/** コンテナーです。 */
		private var containerSprite:Sprite;
		
		/** コンテンツの中身 */
		private var contentBody:Sprite;
		
		/** コンテンツのマスク。 */
		private var contentMask:Sprite;
		
		/** スクロールバーのつまみ */
		private var scrollThumb:Sprite;
		
		/** スクロールバーの座布団。つまみのうごかせる領域に利用される。 */
		private var scrollBase:Sprite;
		
		/** stage の参照 */
		private var stage:Stage;
		
		/** フラグ */
		private var _draggingFlag:Boolean  = false;
		
		/** フラグ */
		private var _scrollingFlag:Boolean = false;
		
		/** スライダーの状態 */
		private var _isSliderVisible:Boolean = false;
		
		/** 前回のマウス座標 */
		private var _pastMouseY:int;
		
		/** Y 方向の力 */
		private var _velocityY:int;
		
		/** スクロールバーのつまみの表示/非表示用のトゥイーンインスタンス */
		private var _tw:Tween;

		/** クリック判定用タイマー */
		private var _timer:Timer;
		
		
		/**
		 * 新しい TouchScrollbarKit インスタンスを作成します。
		 */
		public function TouchScrollbarKit(stage:Stage, 
										containerSprite:Sprite,
										contentBody:Sprite,
										contentMask:Sprite,
										scrollThumb:Sprite,
										scrollBase:Sprite ):void {
			
			this.stage = stage;
			this.containerSprite = containerSprite;
			this.contentBody = contentBody;
			this.contentMask = contentMask;
			this.scrollThumb = scrollThumb;
			this.scrollBase = scrollBase;
			
			//----------------------------------------------------------------------
			//スクロールバーインスタンスの生成
			_scrollbar = new JPPScrollbar(stage);
			
			//スクロールバーの初期化
			_scrollbar.setup(
				contentBody, //スクロール対象となるオブジェクトです．
				"y",         //スクロール対象コンテンツが保持している, スクロールによって実際に変化させたいプロパティ名を表します.
				contentBody.height, //スクロール対象コンテンツの総計サイズを設定します.
				contentMask.height, //スクロール対象コンテンツの表示部分のサイズを設定します.
				0,  //スライダーが上限に達したときの変化対象プロパティの値を設定します.
				0   //スライダーが下限に達したときの変化対象プロパティの値を設定します.
			);
			
			//パーツのバインド
			_scrollbar.base   = scrollBase;      //スクロールエリアとしてバインドするインスタンスを設定します.
			_scrollbar.slider = scrollThumb;    //スライダとしてバインドするインスタンスを設定します.
			
			
			//----------------------------------------------------------------------
			//スライダーおよび対象プロパティの吸着
			
			//スクロール完了時にスライダーをピクセルに吸着させるかどうかを設定します.
			_scrollbar.usePixelFittingSlider = true;
			
			//スクロール完了時に対象プロパティを整数値に吸着させるかどうかを設定します.
			_scrollbar.usePixelFittingContent = true;
			
			
			//----------------------------------------------------------------------
			//スクロールバーの有効化/無効化
			_scrollbar.sliderEnabled = false; //スライダー
			
			
			//----------------------------------------------------------------------
			//マウスホイールの使用/不使用
			_scrollbar.useMouseWheel = true;
			
			
			//----------------------------------------------------------------------
			//オーバーシュート演出
			
			//オーバーシュート(iPhoneのように, 端まで行くとちょっと行き過ぎて戻る演出)を加えるかどうかを切り替えます.
			_scrollbar.useOvershoot = true;
			
			//オーバーシュートを使用する場合, オーバーシュートの最大行き過ぎ量をピクセル数で設定します.
			_scrollbar.overshootPixels = contentMask.height / 4;
			
			//オーバーシュートを使用する場合, オーバーシュートから本来のスクロール座標へ戻る際の緩やかさを設定します.
			//1以上の数値を設定し, 数値が大きくなるほど緩やかに戻るようになります.
			_scrollbar.overshootEasing = 5;
			
			//オーバーシュートを使用する場合, オーバーシュート時にスクロールバーが縮む演出を加えるかどうかを切り替えます.
			_scrollbar.useOvershootDeformationSlider = true;
			
			
			//----------------------------------------------------------------------
			// 初期化
			//スクロールバー、今回は動いた時だけ出る。とりあえず見えないようにしておく
			scrollThumb.alpha = 0;
			
			// スクロールバーの下地。今回は見えないようにする。
			scrollBase.visible = false;
			
			//ドラッグ開始ハンドラの登録
			containerSprite.addEventListener(MouseEvent.MOUSE_DOWN, _startDragHandler);
			
			// 
			_timer = new Timer(60, 1);
			_timer.addEventListener(TimerEvent.TIMER_COMPLETE, timerCompleteHandler);
		}
			
			
		/**
		* 高さの値を更新します。
		*/
		public function update():void{
			
			// 位置を初期位置に戻す
			contentBody.y = 0;
			scrollThumb.y = 0;
			_scrollbar.stopAutoScroll();
			
			//コンテンツの上限値, 下限値
			_scrollbar.contentSize = contentBody.height;
			_scrollbar.maskSize = contentMask.height;
			_scrollbar.upperBound = contentBody.y;
			_scrollbar.lowerBound = contentBody.y - (contentBody.height - contentMask.height);
			
			//コンテンツサイズがマスクサイズに満たない場合の処理
			_scrollbar.slider.visible = !_scrollbar.isUnderFlow;
			
			for(var i:int = 0; i<contentBody.numChildren; i++){
				var obj:Sprite = contentBody.getChildAt(i) as Sprite;
				if(obj){
					obj.addEventListener(MouseEvent.MOUSE_DOWN, handleItemPress);
					obj.addEventListener(MouseEvent.MOUSE_UP, handleItemUp);
				}
			}
			
			_scrollbar.scrollByAbsoluteRatio(0);
			
		}
		
		private var tapItem:Object;
		
		protected function handleItemPress(e:MouseEvent):void
		{
			if(tapItem) {
				tapItem.onUnselect();
			}
			
			tapItem = e.currentTarget as Sprite;
			
			_timer.reset();
			_timer.start();
		}
		private function handleItemUp(e:MouseEvent):void{
			if(tapItem) {
				if(_timer.currentCount == 1){
					tapItem.onTap();
				}
				tapItem.onUnselect();
			}
			_timer.stop();
		}
		
		private function timerCompleteHandler(e:TimerEvent):void{
			if(tapItem) {
				tapItem.onSelect();
			}
		}
		
		/**
		 * ドラッグ開始ハンドラ
		 * @param	e
		 */
		private function _startDragHandler(e:MouseEvent):void {
			containerSprite.removeEventListener(MouseEvent.MOUSE_DOWN, _startDragHandler);
			stage.addEventListener(MouseEvent.MOUSE_UP, _stopDragHandler);
			
			//監視開始
			containerSprite.addEventListener(Event.ENTER_FRAME, _observeHandler);
			_scrollbar.addEventListener(JPPScrollbarEvent.SCROLL_START   , _startScrollHandler);
			_scrollbar.addEventListener(JPPScrollbarEvent.SCROLL_COMPLETE, _stopScrollHandler);
			
			//ドラッグ前処理
			_scrollbar.smoothScrollEasing = 2;
			_pastMouseY = containerSprite.mouseY;
			
			stage.addEventListener(MouseEvent.MOUSE_MOVE, _moveDragHandler);
		}
		
		
		/**
		 * ドラッグ検出ハンドラ
		 * @param	e
		 */
		private function _moveDragHandler(e:MouseEvent):void {
			_draggingFlag = true;
			
			//マウスの動きの差分を計算する
			_velocityY = containerSprite.mouseY - _pastMouseY;
			
			
			if(Math.abs(_velocityY) > 6){
				_timer.stop();
				if(tapItem) {
					tapItem.onUnselect();
					tapItem = null;
				}

				//ドラッグ中の処理
				_scrollbar.scrollByRelativePixel(_velocityY);
				
				//次の計算のために現在のマウス位置を保持
				_pastMouseY = containerSprite.mouseY;

			}
		}
		
		
		/**
		 * ドラッグ終了ハンドラ
		 * @param	e
		 */
		private function _stopDragHandler(e:MouseEvent):void {
			containerSprite.addEventListener(MouseEvent.MOUSE_DOWN, _startDragHandler);
			stage.removeEventListener(MouseEvent.MOUSE_UP, _stopDragHandler);
			stage.removeEventListener(MouseEvent.MOUSE_MOVE, _moveDragHandler);
			
			//ドラッグ操作をしていた場合慣性を働かせる
			if ( _draggingFlag ) {
				_scrollbar.smoothScrollEasing = 5;
				_scrollbar.scrollByRelativePixel(_velocityY * 7);
			}
			
			_draggingFlag = false;
		}
		
		
		/**
		 * スクロール開始ハンドラ
		 * @param	e
		 */
		private function _startScrollHandler(e:JPPScrollbarEvent):void {
			_scrollingFlag = true;
		}
		
		
		/**
		 * スクロール完了ハンドラ
		 * @param	e
		 */
		private function _stopScrollHandler(e:JPPScrollbarEvent):void {
			_scrollingFlag = false;
		}
		
		
		/**
		 * 毎フレーム，マウスの動きとスクロール状況を監視する
		 * @param	e
		 */
		private function _observeHandler(e:Event):void {
			//スライダーを表示する
			if (!_isSliderVisible && (_draggingFlag || _scrollingFlag)) {
				_isSliderVisible = true;
				_showSlider();
			}
			
			//スライダーを非表示にする
			if (_isSliderVisible && (!_draggingFlag && !_scrollingFlag)) {
				_isSliderVisible = false;
				_hideSlider();
				
				//監視終了
				containerSprite.removeEventListener(Event.ENTER_FRAME, _observeHandler);
				_scrollbar.removeEventListener(JPPScrollbarEvent.SCROLL_START   , _startScrollHandler);
				_scrollbar.removeEventListener(JPPScrollbarEvent.SCROLL_COMPLETE, _stopScrollHandler);
			}
		}
		
		
		/**
		 * スライダーを表示する
		 */
		private function _showSlider():void {
			if(_tw) _tw.stop();
			_tw = new Tween(scrollThumb, "alpha", null, scrollThumb.alpha, 1, 0.3, true);
		}
		
		
		/**
		 * スライダーを隠す
		 */
		private function _hideSlider():void {
			if(_tw) _tw.stop();
			_tw = new Tween(scrollThumb, "alpha", null, scrollThumb.alpha, 0, 0.5, true);
		}

	}
}
